# pylint: disable=missing-docstring

def stupid_function(arg): # [too-many-return-statements]
    if arg == 1:
        return 1
    elif arg == 2:
        return 2
    elif arg == 3:
        return 3
    elif arg == 4:
        return 4
    elif arg == 5:
        return 5
    elif arg == 6:
        return 6
    elif arg == 7:
        return 7
    elif arg == 8:
        return 8
    elif arg == 9:
        return 9
    elif arg == 10:
        return 10
    return None


def many_yield(text):
    """Not a problem"""
    if text:
        yield "    line 1: %s\n" % text
        yield "    line 2\n"
        yield "    line 3\n"
        yield "    line 4\n"
        yield "    line 5\n"
    else:
        yield "    line 6\n"
        yield "    line 7\n"
        yield "    line 8\n"
        yield "    line 9\n"
        yield "    line 10\n"



# pylint: disable=missing-docstring, import-error, invalid-name
# pylint: disable=too-few-public-methods, blacklisted-name

import uuid

import foo

from .a import x


class Y(object):
    x = x[0]


def test(default=None):
    return default


class BaseModel(object):
    uuid = test(default=uuid.uuid4)


class bar(object):
    foo = foo.baz



# pylint: disable=missing-docstring,bare-except,pointless-statement,superfluous-parens
def strangeproblem():
    try:
        for _ in range(0, 4):
            message = object()
            print(type(message))
    finally:
        message = object()


try:
    MY_INT = 1
    print("MY_INT = %d" % MY_INT)
finally:
    MY_INT = 2

try:
    pass
except:
    FALSE_POSITIVE = 1
    FALSE_POSITIVE  # here pylint claims used-before-assignment
finally:
    FALSE_POSITIVE = 2  # this line is needed to reproduce the issue



# pylint: disable=too-few-public-methods,import-error, no-absolute-import,missing-docstring
# pylint: disable=useless-super-delegation,wrong-import-position,invalid-name, wrong-import-order

if len('TEST'): # [len-as-condition]
    pass

while not len('TEST'): # [len-as-condition]
    pass

assert len('TEST') > 0 # [len-as-condition]

x = 1 if len('TEST') != 0 else 2 # [len-as-condition]

if len('TEST') == 0: # [len-as-condition]
    pass

if True and len('TEST') == 0: # [len-as-condition]
    pass

if 0 == len('TEST') < 10: # [len-as-condition]
    pass

if 0 < 1 <= len('TEST') < 10: # Should be fine
    pass

if 10 > len('TEST') != 0: # [len-as-condition]
    pass

z = False
if z and len(['T', 'E', 'S', 'T']):  # [len-as-condition]
    pass

if 10 > len('TEST') > 1 > 0:
    pass

f_o_o = len('TEST') or 42  # Should be fine

a = x and len(x)  # Should be fine

if 0 <= len('TEST') < 100:  # Should be fine
    pass

if z or 10 > len('TEST') != 0: # [len-as-condition]
    pass

def some_func():
    return len('TEST') > 0  # Should be fine


def github_issue_1325():
    l = [1, 2, 3]
    length = len(l) if l else 0
    return length


def github_issue_1331(*args):
    assert False, len(args)


def github_issue_1331_v2(*args):
    assert len(args), args  # [len-as-condition]


def github_issue_1331_v3(*args):
    assert len(args) or z, args  # [len-as-condition]



# pylint: disable=missing-docstring,invalid-name,too-few-public-methods

x = 24


def used_before_assignment_1(a):
    if x == a:  # [used-before-assignment]
        for x in [1, 2]:  # [redefined-outer-name]
            pass


def used_before_assignment_2(a):
    if x == a:  # [used-before-assignment]
        pass
    x = 2  # [redefined-outer-name]


def used_before_assignment_3(a):
    if x == a:  # [used-before-assignment]
        if x > 3:
            x = 2  # [redefined-outer-name]


def not_used_before_assignment(a):
    if x == a:
        pass


def not_used_before_assignment_2(a):
    x = 3  # [redefined-outer-name]
    if x == a:
        pass


def func(something):
    return something ** 3


class FalsePositive(object):
    x = func(x)



"""Test that wrong-import-position is properly reset when
other errors are disabled.
"""
# pylint: disable=unused-import, too-few-public-methods


class Something(object):
    """A class before an import."""


import os



# pylint: disable=too-few-public-methods, missing-docstring,no-init

class PluginBase(object):
    subclasses = []

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.subclasses.append(cls)


class Plugin1(PluginBase):
    pass


class Plugin2(PluginBase):
    pass



"""Check for nonlocal and used-before-assignment"""
# pylint: disable=missing-docstring, unused-variable, no-init, too-few-public-methods

__revision__ = 0

def test_ok():
    """ uses nonlocal """
    cnt = 1
    def wrap():
        nonlocal cnt
        cnt = cnt + 1
    wrap()

def test_fail():
    """ doesn't use nonlocal """
    cnt = 1
    def wrap():
        cnt = cnt + 1 # [used-before-assignment]
    wrap()

def test_fail2():
    """ use nonlocal, but for other variable """
    cnt = 1
    count = 1
    def wrap():
        nonlocal count
        cnt = cnt + 1 # [used-before-assignment]
    wrap()

def test_fail3(arg: test_fail4): # [used-before-assignment]
    """ Depends on `test_fail4`, in argument annotation. """
    return arg
# +1: [used-before-assignment, used-before-assignment]
def test_fail4(*args: test_fail5, **kwargs: undefined):
    """ Depends on `test_fail5` and `undefined` in
    variable and named arguments annotations.
    """
    return args, kwargs

def test_fail5()->undefined1: # [used-before-assignment]
    """ Depends on `undefined1` in function return annotation. """

def undefined():
    """ no op """

def undefined1():
    """ no op """



"""Checks static methods are declared with a decorator if within the class
scope and if static method's argument is a member of the class
"""

# pylint: disable=too-few-public-methods, using-constant-test, no-method-argument

class MyClass(object):
    """Some class"""
    def __init__(self):
        pass

    def smethod():
        """static method-to-be"""
    smethod = staticmethod(smethod) # [no-staticmethod-decorator]

    if True:
        smethod = staticmethod(smethod)  # [no-staticmethod-decorator]

    @staticmethod
    def my_second_method():
        """correct static method definition"""

    def other_method():
        """some method"""
    smethod2 = staticmethod(other_method)  # [no-staticmethod-decorator]

def helloworld():
    """says hello"""


MyClass.new_static_method = staticmethod(helloworld)

class MyOtherClass(object):
    """Some other class"""
    _make = staticmethod(tuple.__new__)



# -*- coding: utf-8 -*-
# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2011-2014 Google, Inc.
# Copyright (c) 2012 Tim Hatch <tim@timhatch.com>
# Copyright (c) 2013-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2014 Brett Cannon <brett@python.org>
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
# Copyright (c) 2015 Steven Myint <hg@stevenmyint.com>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Erik <erik.eriksson@yahoo.com>
# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
# Copyright (c) 2017 Martin von Gagern <gagern@google.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Checks for various exception related errors."""

import inspect
import sys

import six
from six.moves import builtins

import astroid
from pylint import checkers
from pylint.checkers import utils
from pylint import interfaces


def _builtin_exceptions():
    def predicate(obj):
        return isinstance(obj, type) and issubclass(obj, BaseException)

    members = inspect.getmembers(six.moves.builtins, predicate)
    return {exc.__name__ for (_, exc) in members}


def _annotated_unpack_infer(stmt, context=None):
    """
    Recursively generate nodes inferred by the given statement.
    If the inferred value is a list or a tuple, recurse on the elements.
    Returns an iterator which yields tuples in the format
    ('original node', 'infered node').
    """
    if isinstance(stmt, (astroid.List, astroid.Tuple)):
        for elt in stmt.elts:
            inferred = utils.safe_infer(elt)
            if inferred and inferred is not astroid.YES:
                yield elt, inferred
        return
    for infered in stmt.infer(context):
        if infered is astroid.YES:
            continue
        yield stmt, infered


PY3K = sys.version_info >= (3, 0)
OVERGENERAL_EXCEPTIONS = ('Exception',)
BUILTINS_NAME = builtins.__name__

MSGS = {
    'E0701': ('Bad except clauses order (%s)',
              'bad-except-order',
              'Used when except clauses are not in the correct order (from the '
              'more specific to the more generic). If you don\'t fix the order, '
              'some exceptions may not be caught by the most specific handler.'),
    'E0702': ('Raising %s while only classes or instances are allowed',
              'raising-bad-type',
              'Used when something which is neither a class, an instance or a \
              string is raised (i.e. a `TypeError` will be raised).'),
    'E0703': ('Exception context set to something which is not an '
              'exception, nor None',
              'bad-exception-context',
              'Used when using the syntax "raise ... from ...", '
              'where the exception context is not an exception, '
              'nor None.',
              {'minversion': (3, 0)}),
    'E0704': ('The raise statement is not inside an except clause',
              'misplaced-bare-raise',
              'Used when a bare raise is not used inside an except clause. '
              'This generates an error, since there are no active exceptions '
              'to be reraised. An exception to this rule is represented by '
              'a bare raise inside a finally clause, which might work, as long '
              'as an exception is raised inside the try block, but it is '
              'nevertheless a code smell that must not be relied upon.'),
    'E0710': ('Raising a new style class which doesn\'t inherit from BaseException',
              'raising-non-exception',
              'Used when a new style class which doesn\'t inherit from \
               BaseException is raised.'),
    'E0711': ('NotImplemented raised - should raise NotImplementedError',
              'notimplemented-raised',
              'Used when NotImplemented is raised instead of \
              NotImplementedError'),
    'E0712': ('Catching an exception which doesn\'t inherit from Exception: %s',
              'catching-non-exception',
              'Used when a class which doesn\'t inherit from \
               Exception is used as an exception in an except clause.'),
    'W0702': ('No exception type(s) specified',
              'bare-except',
              'Used when an except clause doesn\'t specify exceptions type to \
              catch.'),
    'W0703': ('Catching too general exception %s',
              'broad-except',
              'Used when an except catches a too general exception, \
              possibly burying unrelated errors.'),
    'W0705': ('Catching previously caught exception type %s',
              'duplicate-except',
              'Used when an except catches a type that was already caught by '
              'a previous handler.'),
    'W0710': ('Exception doesn\'t inherit from standard "Exception" class',
              'nonstandard-exception',
              'Used when a custom exception class is raised but doesn\'t \
              inherit from the builtin "Exception" class.',
              {'maxversion': (3, 0)}),
    'W0711': ('Exception to catch is the result of a binary "%s" operation',
              'binary-op-exception',
              'Used when the exception to catch is of the form \
              "except A or B:".  If intending to catch multiple, \
              rewrite as "except (A, B):"'),
    'W0715': ('Exception arguments suggest string formatting might be intended',
              'raising-format-tuple',
              'Used when passing multiple arguments to an exception \
              constructor, the first of them a string literal containing what \
              appears to be placeholders intended for formatting'),
    }


class BaseVisitor(object):
    """Base class for visitors defined in this module."""

    def __init__(self, checker, node):
        self._checker = checker
        self._node = node

    def visit(self, node):
        name = node.__class__.__name__.lower()
        dispatch_meth = getattr(self, 'visit_' + name, None)
        if dispatch_meth:
            dispatch_meth(node)
        else:
            self.visit_default(node)

    def visit_default(self, node): # pylint: disable=unused-argument
        """Default implementation for all the nodes."""


class ExceptionRaiseRefVisitor(BaseVisitor):
    """Visit references (anything that is not an AST leaf)."""

    def visit_name(self, name):
        if name.name == 'NotImplemented':
            self._checker.add_message(
                'notimplemented-raised',
                node=self._node)

    def visit_call(self, call):
        if isinstance(call.func, astroid.Name):
            self.visit_name(call.func)
        if (len(call.args) > 1 and
                isinstance(call.args[0], astroid.Const) and
                isinstance(call.args[0].value, six.string_types)):
            msg = call.args[0].value
            if ('%' in msg or
                    ('{' in msg and '}' in msg)):
                self._checker.add_message(
                    'raising-format-tuple',
                    node=self._node)


class ExceptionRaiseLeafVisitor(BaseVisitor):
    """Visitor for handling leaf kinds of a raise value."""

    def visit_const(self, const):
        if not isinstance(const.value, str):
            # raising-string will be emitted from python3 porting checker.
            self._checker.add_message('raising-bad-type', node=self._node,
                                      args=const.value.__class__.__name__)

    def visit_instance(self, instance):
        # pylint: disable=protected-access
        cls = instance._proxied
        self.visit_classdef(cls)

    # Exception instances have a particular class type
    visit_exceptioninstance = visit_instance

    def visit_classdef(self, cls):
        if (not utils.inherit_from_std_ex(cls) and
                utils.has_known_bases(cls)):
            if cls.newstyle:
                self._checker.add_message('raising-non-exception', node=self._node)
            else:
                self._checker.add_message('nonstandard-exception', node=self._node)

    def visit_tuple(self, tuple_node):
        if PY3K or not tuple_node.elts:
            self._checker.add_message('raising-bad-type',
                                      node=self._node,
                                      args='tuple')
            return

        # On Python 2, using the following is not an error:
        #    raise (ZeroDivisionError, None)
        #    raise (ZeroDivisionError, )
        # What's left to do is to check that the first
        # argument is indeed an exception. Verifying the other arguments
        # is not the scope of this check.
        first = tuple_node.elts[0]
        inferred = utils.safe_infer(first)
        if not inferred or inferred is astroid.Uninferable:
            return

        if (isinstance(inferred, astroid.Instance)
                and inferred.__class__.__name__ != 'Instance'):
            # TODO: explain why
            self.visit_default(tuple_node)
        else:
            self.visit(inferred)

    def visit_default(self, node):
        name = getattr(node, 'name', node.__class__.__name__)
        self._checker.add_message('raising-bad-type',
                                  node=self._node,
                                  args=name)


class ExceptionsChecker(checkers.BaseChecker):
    """Exception related checks."""

    __implements__ = interfaces.IAstroidChecker

    name = 'exceptions'
    msgs = MSGS
    priority = -4
    options = (('overgeneral-exceptions',
                {'default' : OVERGENERAL_EXCEPTIONS,
                 'type' : 'csv', 'metavar' : '<comma-separated class names>',
                 'help' : 'Exceptions that will emit a warning '
                          'when being caught. Defaults to "%s"' % (
                              ', '.join(OVERGENERAL_EXCEPTIONS),)}
               ),
              )

    def open(self):
        self._builtin_exceptions = _builtin_exceptions()
        super(ExceptionsChecker, self).open()

    @utils.check_messages('nonstandard-exception', 'misplaced-bare-raise',
                          'raising-bad-type', 'raising-non-exception',
                          'notimplemented-raised', 'bad-exception-context',
                          'raising-format-tuple')
    def visit_raise(self, node):
        if node.exc is None:
            self._check_misplaced_bare_raise(node)
            return

        if PY3K and node.cause:
            self._check_bad_exception_context(node)

        expr = node.exc
        try:
            inferred_value = next(expr.infer())
        except astroid.InferenceError:
            inferred_value = None

        ExceptionRaiseRefVisitor(self, node).visit(expr)

        if inferred_value:
            ExceptionRaiseLeafVisitor(self, node).visit(inferred_value)

    def _check_misplaced_bare_raise(self, node):
        # Filter out if it's present in __exit__.
        scope = node.scope()
        if (isinstance(scope, astroid.FunctionDef)
                and scope.is_method()
                and scope.name == '__exit__'):
            return

        current = node
        # Stop when a new scope is generated or when the raise
        # statement is found inside a TryFinally.
        ignores = (astroid.ExceptHandler, astroid.FunctionDef, astroid.TryFinally)
        while current and not isinstance(current.parent, ignores):
            current = current.parent

        expected = (astroid.ExceptHandler,)
        if not current or not isinstance(current.parent, expected):
            self.add_message('misplaced-bare-raise', node=node)

    def _check_bad_exception_context(self, node):
        """Verify that the exception context is properly set.

        An exception context can be only `None` or an exception.
        """
        cause = utils.safe_infer(node.cause)
        if cause in (astroid.YES, None):
            return

        if isinstance(cause, astroid.Const):
            if cause.value is not None:
                self.add_message('bad-exception-context',
                                 node=node)
        elif (not isinstance(cause, astroid.ClassDef) and
              not utils.inherit_from_std_ex(cause)):
            self.add_message('bad-exception-context',
                             node=node)

    def _check_catching_non_exception(self, handler, exc, part):
        if isinstance(exc, astroid.Tuple):
            # Check if it is a tuple of exceptions.
            inferred = [utils.safe_infer(elt) for elt in exc.elts]
            if any(node is astroid.YES for node in inferred):
                # Don't emit if we don't know every component.
                return
            if all(node and (utils.inherit_from_std_ex(node) or
                             not utils.has_known_bases(node))
                   for node in inferred):
                return

        if not isinstance(exc, astroid.ClassDef):
            # Don't emit the warning if the infered stmt
            # is None, but the exception handler is something else,
            # maybe it was redefined.
            if (isinstance(exc, astroid.Const) and
                    exc.value is None):
                if ((isinstance(handler.type, astroid.Const) and
                     handler.type.value is None) or
                        handler.type.parent_of(exc)):
                    # If the exception handler catches None or
                    # the exception component, which is None, is
                    # defined by the entire exception handler, then
                    # emit a warning.
                    self.add_message('catching-non-exception',
                                     node=handler.type,
                                     args=(part.as_string(), ))
            else:
                self.add_message('catching-non-exception',
                                 node=handler.type,
                                 args=(part.as_string(), ))
            return

        if (not utils.inherit_from_std_ex(exc) and
                exc.name not in self._builtin_exceptions):
            if utils.has_known_bases(exc):
                self.add_message('catching-non-exception',
                                 node=handler.type,
                                 args=(exc.name, ))

    @utils.check_messages('bare-except', 'broad-except',
                          'binary-op-exception', 'bad-except-order',
                          'catching-non-exception', 'duplicate-except')
    def visit_tryexcept(self, node):
        """check for empty except"""
        exceptions_classes = []
        nb_handlers = len(node.handlers)
        for index, handler in enumerate(node.handlers):
            if handler.type is None:
                if not utils.is_raising(handler.body):
                    self.add_message('bare-except', node=handler)
                # check if a "except:" is followed by some other
                # except
                if index < (nb_handlers - 1):
                    msg = 'empty except clause should always appear last'
                    self.add_message('bad-except-order', node=node, args=msg)

            elif isinstance(handler.type, astroid.BoolOp):
                self.add_message('binary-op-exception',
                                 node=handler, args=handler.type.op)
            else:
                try:
                    excs = list(_annotated_unpack_infer(handler.type))
                except astroid.InferenceError:
                    continue

                for part, exc in excs:
                    if exc is astroid.YES:
                        continue
                    if (isinstance(exc, astroid.Instance)
                            and utils.inherit_from_std_ex(exc)):
                        # pylint: disable=protected-access
                        exc = exc._proxied

                    self._check_catching_non_exception(handler, exc, part)

                    if not isinstance(exc, astroid.ClassDef):
                        continue

                    exc_ancestors = [anc for anc in exc.ancestors()
                                     if isinstance(anc, astroid.ClassDef)]

                    for previous_exc in exceptions_classes:
                        if previous_exc in exc_ancestors:
                            msg = '%s is an ancestor class of %s' % (
                                previous_exc.name, exc.name)
                            self.add_message('bad-except-order',
                                             node=handler.type, args=msg)
                    if (exc.name in self.config.overgeneral_exceptions
                            and exc.root().name == utils.EXCEPTIONS_MODULE
                            and not utils.is_raising(handler.body)):
                        self.add_message('broad-except',
                                         args=exc.name, node=handler.type)

                    if exc in exceptions_classes:
                        self.add_message('duplicate-except',
                                         args=exc.name, node=handler.type)

                exceptions_classes += [exc for _, exc in excs]


def register(linter):
    """required method to auto register this checker"""
    linter.register_checker(ExceptionsChecker(linter))



# pylint: disable=missing-docstring, expression-not-assigned, too-few-public-methods, no-member, import-error, no-self-use, line-too-long

from unknown import Unknown


class CustomClass(object):
    def keys(self):
        return []

for key in Unknown().keys():
    pass
for key in Unknown.keys():
    pass
for key in dict.keys():
    pass
for key in {}.values():
    pass
for key in {}.key():
    pass
for key in CustomClass().keys():
    pass

[key for key in {}.keys()] # [consider-iterating-dictionary]
(key for key in {}.keys()) # [consider-iterating-dictionary]
{key for key in {}.keys()} # [consider-iterating-dictionary]
{key: key for key in {}.keys()} # [consider-iterating-dictionary]
COMP1 = [key for key in {}.keys()] # [consider-iterating-dictionary]
COMP2 = (key for key in {}.keys()) # [consider-iterating-dictionary]
COMP3 = {key for key in {}.keys()} # [consider-iterating-dictionary]
COMP4 = {key: key for key in {}.keys()} # [consider-iterating-dictionary]
for key in {}.keys(): # [consider-iterating-dictionary]
    pass

# Issue #1247
DICT = {'a': 1, 'b': 2}
COMP1 = [k * 2 for k in DICT.keys()] + [k * 3 for k in DICT.keys()]  # [consider-iterating-dictionary,consider-iterating-dictionary]
COMP2, COMP3 = [k * 2 for k in DICT.keys()], [k * 3 for k in DICT.keys()]  # [consider-iterating-dictionary,consider-iterating-dictionary]
SOME_TUPLE = ([k * 2 for k in DICT.keys()], [k * 3 for k in DICT.keys()])  # [consider-iterating-dictionary,consider-iterating-dictionary]



"""check operator use"""
# pylint: disable=invalid-name, pointless-statement
a = 1
a += 5
a = +a
b = ++a # [nonexistent-operator]
++a # [nonexistent-operator]
c = (++a) * b # [nonexistent-operator]

a = 1
a -= 5
b = --a # [nonexistent-operator]
b = a
--a # [nonexistent-operator]
c = (--a) * b # [nonexistent-operator]



# Copyright (c) 2016 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2016 Glenn Matthews <glmatthe@cisco.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import six

import astroid
from pylint.checkers import BaseChecker
from pylint.checkers.utils import check_messages, is_none, node_type
from pylint.interfaces import IAstroidChecker


BUILTINS = six.moves.builtins.__name__


class MultipleTypesChecker(BaseChecker):
    """Checks for variable type redefinitions (NoneType excepted)

    At a function, method, class or module scope

    This rule could be improved:

    - Currently, if an attribute is set to different types in 2 methods of a
      same class, it won't be detected (see functional test)
    - One could improve the support for inference on assignment with tuples,
      ifexpr, etc. Also it would be great to have support for inference on
      str.split()
    """
    __implements__ = IAstroidChecker

    name = 'multiple_types'
    msgs = {'R0204': ('Redefinition of %s type from %s to %s',
                      'redefined-variable-type',
                      'Used when the type of a variable changes inside a '
                      'method or a function.'
                     ),
           }

    def visit_classdef(self, _):
        self._assigns.append({})

    @check_messages('redefined-variable-type')
    def leave_classdef(self, _):
        self._check_and_add_messages()

    visit_functiondef = visit_classdef
    leave_functiondef = leave_module = leave_classdef

    def visit_module(self, _):
        self._assigns = [{}]

    def _check_and_add_messages(self):
        assigns = self._assigns.pop()
        for name, args in assigns.items():
            if len(args) <= 1:
                continue
            orig_node, orig_type = args[0]
            # Check if there is a type in the following nodes that would be
            # different from orig_type.
            for redef_node, redef_type in args[1:]:
                if redef_type == orig_type:
                    continue
                # if a variable is defined to several types in a if node,
                # this is not actually redefining.
                orig_parent = orig_node.parent
                redef_parent = redef_node.parent
                if isinstance(orig_parent, astroid.If):
                    if orig_parent == redef_parent:
                        if (redef_node in orig_parent.orelse and
                                orig_node not in orig_parent.orelse):
                            orig_node, orig_type = redef_node, redef_type
                            continue
                    elif (isinstance(redef_parent, astroid.If) and
                          redef_parent in orig_parent.nodes_of_class(astroid.If)):
                        orig_node, orig_type = redef_node, redef_type
                        continue
                orig_type = orig_type.replace(BUILTINS + ".", '')
                redef_type = redef_type.replace(BUILTINS + ".", '')
                self.add_message('redefined-variable-type', node=redef_node,
                                 args=(name, orig_type, redef_type))
                break

    def visit_assign(self, node):
        # we don't handle multiple assignment nor slice assignment
        target = node.targets[0]
        if isinstance(target, (astroid.Tuple, astroid.Subscript)):
            return
        # ignore NoneType
        if is_none(node):
            return
        _type = node_type(node.value)
        if _type:
            self._assigns[-1].setdefault(target.as_string(), []).append(
                (node, _type.pytype()))


def register(linter):
    """Required method to auto register this checker.

    :param linter: Main interface object for Pylint plugins
    :type linter: Pylint object
    """
    linter.register_checker(MultipleTypesChecker(linter))



# -*- coding: utf-8 -*-
# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2014-2015 Brett Cannon <brett@python.org>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2015 Cosmin Poieana <cmin@ropython.org>
# Copyright (c) 2015 Viorel Stirbu <viorels@gmail.com>
# Copyright (c) 2016-2017 Roy Williams <roy.williams.iii@gmail.com>
# Copyright (c) 2016 Roy Williams <rwilliams@lyft.com>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Tests for the python3 checkers."""
from __future__ import absolute_import

import sys
import textwrap

import pytest

import astroid

from pylint import testutils
from pylint.checkers import python3 as checker
from pylint.interfaces import INFERENCE_FAILURE, INFERENCE


# Decorator for any tests that will fail under Python 3
python2_only = pytest.mark.skipif(sys.version_info[0] > 2, reason='Python 2 only')

# TODO(cpopa): Port these to the functional test framework instead.

class TestPython3Checker(testutils.CheckerTestCase):
    CHECKER_CLASS = checker.Python3Checker

    def check_bad_builtin(self, builtin_name):
        node = astroid.extract_node(builtin_name + '  #@')
        message = builtin_name.lower() + '-builtin'
        with self.assertAddsMessages(testutils.Message(message, node=node)):
            self.checker.visit_name(node)

    @python2_only
    def test_bad_builtins(self):
        builtins = [
            'apply',
            'buffer',
            'cmp',
            'coerce',
            'execfile',
            'file',
            'input',
            'intern',
            'long',
            'raw_input',
            'round',
            'reduce',
            'StandardError',
            'unichr',
            'unicode',
            'xrange',
            'reload',
        ]
        for builtin in builtins:
            self.check_bad_builtin(builtin)

    def as_iterable_in_for_loop_test(self, fxn):
        code = "for x in {}(): pass".format(fxn)
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    def as_used_by_iterable_in_for_loop_test(self, fxn):
        checker = '{}-builtin-not-iterating'.format(fxn)
        node = astroid.extract_node("""
        for x in (whatever(
            {}() #@
        )):
            pass
        """.format(fxn))
        message = testutils.Message(checker, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def as_iterable_in_genexp_test(self, fxn):
        code = "x = (x for x in {}())".format(fxn)
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    def as_iterable_in_listcomp_test(self, fxn):
        code = "x = [x for x in {}(None, [1])]".format(fxn)
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    def as_used_in_variant_in_genexp_test(self, fxn):
        checker = '{}-builtin-not-iterating'.format(fxn)
        node = astroid.extract_node("""
        list(
            __({}(x))
            for x in [1]
        )
        """.format(fxn))
        message = testutils.Message(checker, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def as_used_in_variant_in_listcomp_test(self, fxn):
        checker = '{}-builtin-not-iterating'.format(fxn)
        node = astroid.extract_node("""
        [
            __({}(None, x))
        for x in [[1]]]
        """.format(fxn))
        message = testutils.Message(checker, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def as_argument_to_callable_constructor_test(self, fxn, callable_fn):
        module = astroid.parse("x = {}({}())".format(callable_fn, fxn))
        with self.assertNoMessages():
            self.walk(module)

    def as_argument_to_materialized_filter(self, callable_fn):
        module = astroid.parse("list(filter(None, {}()))".format(callable_fn))
        with self.assertNoMessages():
            self.walk(module)

    def as_argument_to_random_fxn_test(self, fxn):
        checker = '{}-builtin-not-iterating'.format(fxn)
        node = astroid.extract_node("""
        y(
            {}() #@
        )
        """.format(fxn))
        message = testutils.Message(checker, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def as_argument_to_str_join_test(self, fxn):
        code = "x = ''.join({}())".format(fxn)
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    def as_iterable_in_unpacking(self, fxn):
        node = astroid.extract_node("""
        a, b = __({}())
        """.format(fxn))
        with self.assertNoMessages():
            self.checker.visit_call(node)

    def as_assignment(self, fxn):
        checker = '{}-builtin-not-iterating'.format(fxn)
        node = astroid.extract_node("""
        a = __({}())
        """.format(fxn))
        message = testutils.Message(checker, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def iterating_context_tests(self, fxn):
        """Helper for verifying a function isn't used as an iterator."""
        self.as_iterable_in_for_loop_test(fxn)
        self.as_used_by_iterable_in_for_loop_test(fxn)
        self.as_iterable_in_genexp_test(fxn)
        self.as_iterable_in_listcomp_test(fxn)
        self.as_used_in_variant_in_genexp_test(fxn)
        self.as_used_in_variant_in_listcomp_test(fxn)
        self.as_argument_to_random_fxn_test(fxn)
        self.as_argument_to_str_join_test(fxn)
        self.as_iterable_in_unpacking(fxn)
        self.as_assignment(fxn)
        self.as_argument_to_materialized_filter(fxn)

        for func in ('iter', 'list', 'tuple', 'sorted',
                     'set', 'sum', 'any', 'all',
                     'enumerate', 'dict'):
            self.as_argument_to_callable_constructor_test(fxn, func)

    @python2_only
    def test_dict_methods_in_iterating_context(self):
        iterating_code = [
            'for x in {}: pass',
            '(x for x in {})',
            '[x for x in {}]',
            'func({})',
            'a, b = {}'
        ]
        non_iterating_code = [
            'x = __({}())',
            '__({}())[0]'
        ]

        for method in ('keys', 'items', 'values'):
            dict_method = '{{}}.{}'.format(method)

            for code in iterating_code:
                with_value = code.format(dict_method)
                module = astroid.parse(with_value)
                with self.assertNoMessages():
                    self.walk(module)

            for code in non_iterating_code:
                with_value = code.format(dict_method)
                node = astroid.extract_node(with_value)

                checker = 'dict-{}-not-iterating'.format(method)
                message = testutils.Message(checker, node=node)
                with self.assertAddsMessages(message):
                    self.checker.visit_call(node)

    @python2_only
    def test_map_in_iterating_context(self):
        self.iterating_context_tests('map')

    @python2_only
    def test_zip_in_iterating_context(self):
        self.iterating_context_tests('zip')

    @python2_only
    def test_range_in_iterating_context(self):
        self.iterating_context_tests('range')

    @python2_only
    def test_filter_in_iterating_context(self):
        self.iterating_context_tests('filter')

    def defined_method_test(self, method, warning):
        """Helper for verifying that a certain method is not defined."""
        node = astroid.extract_node("""
            class Foo(object):
                def __{0}__(self, other):  #@
                    pass""".format(method))
        message = testutils.Message(warning, node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_functiondef(node)

    def test_delslice_method(self):
        self.defined_method_test('delslice', 'delslice-method')

    def test_getslice_method(self):
        self.defined_method_test('getslice', 'getslice-method')

    def test_setslice_method(self):
        self.defined_method_test('setslice', 'setslice-method')

    def test_coerce_method(self):
        self.defined_method_test('coerce', 'coerce-method')

    def test_oct_method(self):
        self.defined_method_test('oct', 'oct-method')

    def test_hex_method(self):
        self.defined_method_test('hex', 'hex-method')

    def test_nonzero_method(self):
        self.defined_method_test('nonzero', 'nonzero-method')

    def test_cmp_method(self):
        self.defined_method_test('cmp', 'cmp-method')

    def test_div_method(self):
        self.defined_method_test('div', 'div-method')

    def test_idiv_method(self):
        self.defined_method_test('idiv', 'idiv-method')

    def test_rdiv_method(self):
        self.defined_method_test('rdiv', 'rdiv-method')

    def test_eq_and_hash_method(self):
        """Helper for verifying that a certain method is not defined."""
        node = astroid.extract_node("""
            class Foo(object):  #@
                def __eq__(self, other):
                    pass
                def __hash__(self):
                    pass""")
        with self.assertNoMessages():
            self.checker.visit_classdef(node)

    def test_eq_and_hash_is_none(self):
        """Helper for verifying that a certain method is not defined."""
        node = astroid.extract_node("""
            class Foo(object):  #@
                def __eq__(self, other):
                    pass
                __hash__ = None""")
        with self.assertNoMessages():
            self.checker.visit_classdef(node)

    def test_eq_without_hash_method(self):
        """Helper for verifying that a certain method is not defined."""
        node = astroid.extract_node("""
            class Foo(object):  #@
                def __eq__(self, other):
                    pass""")
        message = testutils.Message('eq-without-hash', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_classdef(node)

    @python2_only
    def test_print_statement(self):
        node = astroid.extract_node('print "Hello, World!" #@')
        message = testutils.Message('print-statement', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_print(node)

    @python2_only
    def test_backtick(self):
        node = astroid.extract_node('`test`')
        message = testutils.Message('backtick', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_repr(node)

    def test_relative_import(self):
        node = astroid.extract_node('import string  #@')
        message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_import(node)
        with self.assertNoMessages():
            # message should only be added once
            self.checker.visit_import(node)

    def test_relative_from_import(self):
        node = astroid.extract_node('from os import path  #@')
        message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_importfrom(node)
        with self.assertNoMessages():
            # message should only be added once
            self.checker.visit_importfrom(node)

    def test_absolute_import(self):
        module_import = astroid.parse(
            'from __future__ import absolute_import; import os')
        module_from = astroid.parse(
            'from __future__ import absolute_import; from os import path')
        with self.assertNoMessages():
            for module in (module_import, module_from):
                self.walk(module)

    def test_import_star_module_level(self):
        node = astroid.extract_node('''
        def test():
            from lala import * #@
        ''')
        absolute = testutils.Message('no-absolute-import', node=node)
        star = testutils.Message('import-star-module-level', node=node)
        with self.assertAddsMessages(absolute, star):
            self.checker.visit_importfrom(node)

    def test_division(self):
        node = astroid.extract_node('3 / 2  #@')
        message = testutils.Message('old-division', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_binop(node)

    def test_division_with_future_statement(self):
        module = astroid.parse('from __future__ import division; 3 / 2')
        with self.assertNoMessages():
            self.walk(module)

    def test_floor_division(self):
        node = astroid.extract_node(' 3 // 2  #@')
        with self.assertNoMessages():
            self.checker.visit_binop(node)

    def test_division_by_float(self):
        left_node = astroid.extract_node('3.0 / 2 #@')
        right_node = astroid.extract_node(' 3 / 2.0  #@')
        with self.assertNoMessages():
            for node in (left_node, right_node):
                self.checker.visit_binop(node)

    def test_dict_iter_method(self):
        for meth in ('keys', 'values', 'items'):
            node = astroid.extract_node('x.iter%s()  #@' % meth)
            message = testutils.Message('dict-iter-method', node=node)
            with self.assertAddsMessages(message):
                self.checker.visit_call(node)

    def test_dict_iter_method_on_dict(self):
        node = astroid.extract_node('{}.iterkeys()')
        message = testutils.Message('dict-iter-method', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def test_dict_not_iter_method(self):
        arg_node = astroid.extract_node('x.iterkeys(x)  #@')
        stararg_node = astroid.extract_node('x.iterkeys(*x)  #@')
        kwarg_node = astroid.extract_node('x.iterkeys(y=x)  #@')
        non_dict_node = astroid.extract_node('x=[]\nx.iterkeys() #@')
        with self.assertNoMessages():
            for node in (arg_node, stararg_node, kwarg_node, non_dict_node):
                self.checker.visit_call(node)

    def test_dict_view_method(self):
        for meth in ('keys', 'values', 'items'):
            node = astroid.extract_node('x.view%s()  #@' % meth)
            message = testutils.Message('dict-view-method', node=node)
            with self.assertAddsMessages(message):
                self.checker.visit_call(node)

    def test_dict_view_method_on_dict(self):
        node = astroid.extract_node('{}.viewkeys()')
        message = testutils.Message('dict-view-method', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def test_dict_not_view_method(self):
        arg_node = astroid.extract_node('x.viewkeys(x)  #@')
        stararg_node = astroid.extract_node('x.viewkeys(*x)  #@')
        kwarg_node = astroid.extract_node('x.viewkeys(y=x)  #@')
        non_dict_node = astroid.extract_node('x=[]\nx.viewkeys() #@')
        with self.assertNoMessages():
            for node in (arg_node, stararg_node, kwarg_node, non_dict_node):
                self.checker.visit_call(node)

    def test_next_method(self):
        node = astroid.extract_node('x.next()  #@')
        message = testutils.Message('next-method-called', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    def test_not_next_method(self):
        arg_node = astroid.extract_node('x.next(x)  #@')
        stararg_node = astroid.extract_node('x.next(*x)  #@')
        kwarg_node = astroid.extract_node('x.next(y=x)  #@')
        with self.assertNoMessages():
            for node in (arg_node, stararg_node, kwarg_node):
                self.checker.visit_call(node)

    def test_metaclass_assignment(self):
        node = astroid.extract_node("""
            class Foo(object):  #@
                __metaclass__ = type""")
        message = testutils.Message('metaclass-assignment', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_classdef(node)

    def test_metaclass_global_assignment(self):
        module = astroid.parse('__metaclass__ = type')
        with self.assertNoMessages():
            self.walk(module)

    @python2_only
    def test_parameter_unpacking(self):
        node = astroid.extract_node('def func((a, b)):#@\n pass')
        arg = node.args.args[0]
        with self.assertAddsMessages(testutils.Message('parameter-unpacking', node=arg)):
            self.checker.visit_arguments(node.args)

    @python2_only
    def test_old_raise_syntax(self):
        node = astroid.extract_node('raise Exception, "test"')
        message = testutils.Message('old-raise-syntax', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_raise(node)

    @python2_only
    def test_exception_message_attribute(self):
        node = astroid.extract_node("""
        try:
            raise Exception("test")
        except Exception as e:
            e.message #@
        """)
        message = testutils.Message('exception-message-attribute', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_attribute(node)

    @python2_only
    def test_normal_message_attribute(self):
        node = astroid.extract_node("""
        e.message #@
        """)
        with self.assertNoMessages():
            self.checker.visit_attribute(node)

    @python2_only
    def test_invalid_codec(self):
        node = astroid.extract_node('foobar.encode("hex") #@')
        message = testutils.Message('invalid-str-codec', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_valid_codec(self):
        node = astroid.extract_node('foobar.encode("ascii", "ignore")  #@')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_visit_call_with_kwarg(self):
        node = astroid.extract_node('foobar.raz(encoding="hex")  #@')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_invalid_open_codec(self):
        node = astroid.extract_node('open(foobar, encoding="hex") #@')
        message = testutils.Message('invalid-str-codec', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_valid_open_codec(self):
        node = astroid.extract_node('open(foobar, encoding="palmos") #@')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_raising_string(self):
        node = astroid.extract_node('raise "Test"')
        message = testutils.Message('raising-string', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_raise(node)

    @python2_only
    def test_checker_disabled_by_default(self):
        node = astroid.parse(textwrap.dedent("""
        abc = 1l
        raise Exception, "test"
        raise "test"
        `abc`
        """))
        with self.assertNoMessages():
            self.walk(node)

    def test_using_cmp_argument(self):
        nodes = astroid.extract_node("""
        [].sort(cmp=lambda x: x) #@
        a = list(range(x))
        a.sort(cmp=lambda x: x) #@

        sorted([], cmp=lambda x: x) #@
        """)
        for node in nodes:
            message = testutils.Message('using-cmp-argument', node=node)
            with self.assertAddsMessages(message):
                self.checker.visit_call(node)

    @python2_only
    def test_sys_maxint(self):
        node = astroid.extract_node('''
        import sys
        sys.maxint #@
        ''')
        message = testutils.Message('sys-max-int', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_attribute(node)

    @python2_only
    def test_itertools_izip(self):
        node = astroid.extract_node('''
        from itertools import izip #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('deprecated-itertools-function', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_deprecated_types_fiels(self):
        node = astroid.extract_node('''
        from types import StringType #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('deprecated-types-field', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_sys_maxint_imort_from(self):
        node = astroid.extract_node('''
        from sys import maxint #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('sys-max-int', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_object_maxint(self):
        node = astroid.extract_node('''
        sys = object()
        sys.maxint #@
        ''')
        with self.assertNoMessages():
            self.checker.visit_attribute(node)

    @python2_only
    def test_bad_import(self):
        node = astroid.extract_node('''
        import urllib2, sys #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('bad-python3-import', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_import(node)

    @python2_only
    def test_bad_import_not_on_relative(self):
        samples = [
            'from .commands import titi',
            'from . import commands',
        ]
        for code in samples:
            node = astroid.extract_node(code)
            absolute_import_message = testutils.Message('no-absolute-import', node=node)
            with self.assertAddsMessages(absolute_import_message):
                self.checker.visit_importfrom(node)
            self.checker._future_absolute_import = False

    @python2_only
    def test_bad_import_conditional(self):
        node = astroid.extract_node('''
        import six
        if six.PY2:
            import urllib2 #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(absolute_import_message):
            self.checker.visit_import(node)

    @python2_only
    def test_bad_import_try_except_handler(self):
        node = astroid.extract_node('''
        try:
            from hashlib import sha
        except:
            import sha #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(absolute_import_message):
            self.checker.visit_import(node)

    @python2_only
    def test_bad_import_try(self):
        node = astroid.extract_node('''
        try:
            import md5  #@
        except:
            from hashlib import md5
        finally:
            pass
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(absolute_import_message):
            self.checker.visit_import(node)

    @python2_only
    def test_bad_import_try_finally(self):
        node = astroid.extract_node('''
        try:
            import Queue  #@
        finally:
            import queue
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('bad-python3-import', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_import(node)

    @python2_only
    def test_bad_import_from(self):
        node = astroid.extract_node('''
        from cStringIO import StringIO #@
        ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('bad-python3-import', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_bad_string_attribute(self):
        node = astroid.extract_node('''
        import string
        string.maketrans #@
        ''')
        message = testutils.Message('deprecated-string-function', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_attribute(node)

    @python2_only
    def test_ok_string_attribute(self):
        node = astroid.extract_node('''
        import string
        string.ascii_letters #@
        ''')
        with self.assertNoMessages():
            self.checker.visit_attribute(node)

    @python2_only
    def test_bad_string_call(self):
        node = astroid.extract_node('''
        import string
        string.upper("hello world") #@
        ''')
        message = testutils.Message('deprecated-string-function', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_ok_shadowed_call(self):
        node = astroid.extract_node('''
        import six.moves.configparser
        six.moves.configparser.ConfigParser() #@
        ''')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_ok_string_call(self):
        node = astroid.extract_node('''
        import string
        string.Foramtter() #@
        ''')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_bad_string_import_from(self):
        node = astroid.extract_node('''
         from string import atoi #@
         ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        message = testutils.Message('deprecated-string-function', node=node)
        with self.assertAddsMessages(absolute_import_message, message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_ok_string_import_from(self):
        node = astroid.extract_node('''
         from string import digits #@
         ''')
        absolute_import_message = testutils.Message('no-absolute-import', node=node)
        with self.assertAddsMessages(absolute_import_message):
            self.checker.visit_importfrom(node)

    @python2_only
    def test_bad_str_translate_call_string_literal(self):
        node = astroid.extract_node('''
         foobar.translate(None, 'abc123') #@
         ''')
        message = testutils.Message('deprecated-str-translate-call', node=node,
                                    confidence=INFERENCE_FAILURE)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_bad_str_translate_call_variable(self):
        node = astroid.extract_node('''
         def raz(foobar):
           foobar.translate(None, 'hello') #@
         ''')
        message = testutils.Message('deprecated-str-translate-call', node=node,
                                    confidence=INFERENCE_FAILURE)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_bad_str_translate_call_infer_str(self):
        node = astroid.extract_node('''
         foobar = "hello world"
         foobar.translate(None, foobar) #@
         ''')
        message = testutils.Message('deprecated-str-translate-call', node=node,
                                    confidence=INFERENCE)
        with self.assertAddsMessages(message):
            self.checker.visit_call(node)

    @python2_only
    def test_ok_str_translate_call_integer(self):
        node = astroid.extract_node('''
         foobar.translate(None, 33) #@
         ''')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_ok_str_translate_call_keyword(self):
        node = astroid.extract_node('''
         foobar.translate(None, 'foobar', raz=33) #@
         ''')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    @python2_only
    def test_ok_str_translate_call_not_str(self):
        node = astroid.extract_node('''
         foobar = {}
         foobar.translate(None, 'foobar') #@
         ''')
        with self.assertNoMessages():
            self.checker.visit_call(node)

    def test_non_py2_conditional(self):
        code = '''
        from __future__ import absolute_import
        import sys
        x = {}
        if sys.maxsize:
            x.iterkeys()  #@
        '''
        node = astroid.extract_node(code)
        module = node.parent.parent
        message = testutils.Message('dict-iter-method', node=node)
        with self.assertAddsMessages(message):
            self.walk(module)

    def test_six_conditional(self):
        code = '''
        from __future__ import absolute_import
        import six
        x = {}
        if six.PY2:
            x.iterkeys()
        '''
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    @python2_only
    def test_versioninfo_conditional(self):
        code = '''
        from __future__ import absolute_import
        import sys
        x = {}
        if sys.version_info[0] == 2:
            x.iterkeys()
        '''
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    @python2_only
    def test_versioninfo_tuple_conditional(self):
        code = '''
        from __future__ import absolute_import
        import sys
        x = {}
        if sys.version_info == (2, 7):
            x.iterkeys()
        '''
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    @python2_only
    def test_six_ifexp_conditional(self):
        code = '''
        from __future__ import absolute_import
        import six
        import string
        string.translate if six.PY2 else None
        '''
        module = astroid.parse(code)
        with self.assertNoMessages():
            self.walk(module)

    @python2_only
    def test_next_defined(self):
        node = astroid.extract_node("""
            class Foo(object):
                def next(self):  #@
                    pass""")
        message = testutils.Message('next-method-defined', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_functiondef(node)

    @python2_only
    def test_next_defined_too_many_args(self):
        node = astroid.extract_node("""
            class Foo(object):
                def next(self, foo=None):  #@
                    pass""")
        with self.assertNoMessages():
            self.checker.visit_functiondef(node)

    @python2_only
    def test_next_defined_static_method_too_many_args(self):
        node = astroid.extract_node("""
            class Foo(object):
                @staticmethod
                def next(self):  #@
                    pass""")
        with self.assertNoMessages():
            self.checker.visit_functiondef(node)

    @python2_only
    def test_next_defined_static_method(self):
        node = astroid.extract_node("""
            class Foo(object):
                @staticmethod
                def next():  #@
                    pass""")
        message = testutils.Message('next-method-defined', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_functiondef(node)

    @python2_only
    def test_next_defined_class_method(self):
        node = astroid.extract_node("""
            class Foo(object):
                @classmethod
                def next(cls):  #@
                    pass""")
        message = testutils.Message('next-method-defined', node=node)
        with self.assertAddsMessages(message):
            self.checker.visit_functiondef(node)


@python2_only
class TestPython3TokenChecker(testutils.CheckerTestCase):

    CHECKER_CLASS = checker.Python3TokenChecker

    def _test_token_message(self, code, symbolic_message):
        tokens = testutils._tokenize_str(code)
        message = testutils.Message(symbolic_message, line=1)
        with self.assertAddsMessages(message):
            self.checker.process_tokens(tokens)

    def test_long_suffix(self):
        for code in ("1l", "1L"):
            self._test_token_message(code, 'long-suffix')

    def test_old_ne_operator(self):
        self._test_token_message("1 <> 2", "old-ne-operator")

    def test_old_octal_literal(self):
        for octal in ("045", "055", "075", "077", "076543"):
            self._test_token_message(octal, "old-octal-literal")

        # Make sure we are catching only octals.
        for non_octal in ("45", "00", "085", "08", "1"):
            tokens = testutils._tokenize_str(non_octal)
            with self.assertNoMessages():
                self.checker.process_tokens(tokens)

    def test_non_ascii_bytes_literal(self):
        code = 'b"测试"'
        self._test_token_message(code, 'non-ascii-bytes-literal')
        for code in ("测试", u"测试", u'abcdef', b'\x80'):
            tokens = testutils._tokenize_str(code)
            with self.assertNoMessages():
                self.checker.process_tokens(tokens)



"""Test that valid class attribute doesn't trigger errors"""
__revision__ = 'sponge bob'

class Clazz(object):
    "dummy class"

    def __init__(self):
        self.topic = 5
        self._data = 45

    def change_type(self, new_class):
        """Change type"""
        self.__class__ = new_class

    def do_nothing(self):
        "I do nothing useful"
        return self.topic + 56
